MODULE diag_header
!-------------------------------------------------------------------------------
!
!     Header for data diagnostics
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  implicit none

  public

  integer, parameter :: DP = selected_real_kind(14)
  integer, parameter :: nhead = 4  ! Byte sizes of header and footer
                                   ! in sequential binary files
  integer, parameter :: nmom = 6   ! Number of output moments
  integer, parameter :: ntrn = 6   ! Number of output transfer diagnostics


!%%% DIAG parameters %%%
  integer, parameter :: snum = 1      ! begining of simulation runs
  integer, parameter :: enum = 1      ! end of simulation runs
!%%%%%%%%%%%%%%%%%%%%%%

!%%% GKV parameters %%%
  integer, parameter :: nxw = 128, nyw = 32
  integer, parameter :: nx = 80, global_ny = 19  ! 2/3 de-aliasing rule
  integer, parameter :: global_nz = 24, global_nv = 32, global_nm = 15

  integer, parameter :: nzb = 3, &  ! the number of ghost grids in z 
                        nvb = 3     ! the number of ghost grids in v and m

  integer, parameter :: nprocw = 2, nprocz = 6, nprocv = 8, nprocm = 4, nprocs = 2
!%%%%%%%%%%%%%%%%%%%%%%


  integer, parameter :: nxw_size = (2*nxw-1)/nprocw     ! local allocation size (0:nxw_size)
  integer, parameter :: ny       = global_ny / nprocw   ! local allocation size (0:ny)
  integer, parameter :: nz = global_nz / nprocz,          &
                        nv = global_nv / nprocv,          &
                        nm = (global_nm + 1) / nprocm - 1,&
                        ns = nprocs
  integer, parameter :: nproc = nprocw * nprocz * nprocv * nprocm * nprocs

! ---- y dimension -------
  integer :: ist_y(0:nprocw-1)               ! local start index of y
  integer :: iend_y(0:nprocw-1)              ! local end   index of y
  integer :: nsize_y(0:nprocw-1)             ! local size of y
  integer :: ist1_y(0:nprocw-1)              ! local start index of y for global start index 1 

  integer :: ist_y_g(0:nprocw-1)             ! global start index of y
  integer :: iend_y_g(0:nprocw-1)            ! global end   index of y

  real(kind=DP) :: tend                            ! end time
  real(kind=DP) :: dtout_fxv, dtout_ptn, dtout_eng ! time-spacing for output
  real(kind=DP) :: dtout_dtc                       ! time-spacing for dt control

  real(kind=DP),    parameter :: pi  = 3.141592653589793_DP, twopi = pi * 2._DP
  real(kind=DP),    parameter :: eps = 0.0000000001_DP
  complex(kind=DP), parameter :: ui  = ( 0._DP, 1._DP )


!--- unit number for file I/O ---
  integer, parameter ::  &
              olog = 10, &
           offinzv = 20, pffinzv = 21, &
          offinzvm = 22, &
        omominkxky = 40, &
          omominxy = 42, &
          omominxz = 44, &
           omominz = 46, &
         omominxyz = 48, &
        omominfreq = 50, &
        ocorrelate = 52, &
      obicoherence = 54, &
          ophireal = 56, &
       ozfshearing = 58, &
           ozfsave = 60, &
        ozfdensity = 62, &
        otrninkxky = 70, &
     oflttrninkxky = 72, &
     oflstrninkxky = 50000000, &
     ofldtrninkxky = 60000000


END MODULE diag_header
