MODULE out_mominkxky
!-------------------------------------------------------------------------------
!
!     Output moments in (x,y)
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_header
  implicit none

  private

  public phiinkxky, Alinkxky, mominkxky, fluxinkxky, mominkxky_all, fpintinkxky


 CONTAINS


SUBROUTINE phiinkxky( loop )
!-------------------------------------------------------------------------------
!
!     Output phi in (kx,ky) at loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_loop
  use diag_geom, only : kx, gky, grootg, cfsrf, fct_e_energy

  integer, intent(in) :: loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: phi
  real(kind=DP), dimension(-nx:nx,0:global_ny) :: phi2, e_energy
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz

    call rb_phi_gettime( loop, time )
    call rb_phi_loop( loop, phi )

    phi2(:,:) = 0._DP
    e_energy(:,:) = 0._DP
    do iz = -global_nz, global_nz-1
      fct = grootg(iz) / cfsrf      ! flux-surface average
      do my = 0, global_ny
        do mx = -nx, nx
          phi2(mx,my) = phi2(mx,my) + fct * abs(phi(mx,my,iz))**2
          e_energy(mx,my) = e_energy(mx,my)  &
                          + fct * fct_e_energy(mx,my,iz) * abs(phi(mx,my,iz))**2
        end do
      end do
    end do

    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/phiinkxky_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(99a17)" ) "#              kx","ky","<|phi|^2>","W_E"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my), phi2(mx,my), e_energy(mx,my)
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE phiinkxky


SUBROUTINE Alinkxky( loop )
!-------------------------------------------------------------------------------
!
!     Output Al in (kx,ky) at loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_Al_gettime, rb_Al_loop
  use diag_geom, only : kx, gky, grootg, cfsrf, fct_m_energy

  integer, intent(in) :: loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: Al
  real(kind=DP), dimension(-nx:nx,0:global_ny) :: Al2, m_energy
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz

    call rb_Al_gettime( loop, time )
    call rb_Al_loop( loop, Al )

    Al2(:,:) = 0._DP
    m_energy(:,:) = 0._DP
    do iz = -global_nz, global_nz-1
      fct = grootg(iz) / cfsrf      ! flux-surface average
      do my = 0, global_ny
        do mx = -nx, nx
          Al2(mx,my) = Al2(mx,my) + fct * abs(Al(mx,my,iz))**2
          m_energy(mx,my) = m_energy(mx,my)  &
                          + fct * fct_m_energy(mx,my,iz) * abs(Al(mx,my,iz))**2
        end do
      end do
    end do

    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/Alinkxky_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(99a17)" ) "#              kx","ky","<|Al|^2>","W_E"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my), Al2(mx,my), m_energy(mx,my)
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE Alinkxky


SUBROUTINE mominkxky( imom, is, loop )
!-------------------------------------------------------------------------------
!
!     Output moments in (kx,ky) at imom, is, loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_mom_gettime, rb_mom_imomisloop
  use diag_geom, only : kx, gky, grootg, cfsrf

  integer, intent(in) :: imom, is, loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: mom
  real(kind=DP), dimension(-nx:nx,0:global_ny) :: mom2
  character(len=1) :: cimom, cis
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz

    call rb_mom_gettime( loop, time )
    call rb_mom_imomisloop( imom, is, loop, mom )

    mom2(:,:) = 0._DP
    do iz = -global_nz, global_nz-1
      fct = grootg(iz) / cfsrf      ! flux-surface average
      do my = 0, global_ny
        do mx = -nx, nx
          mom2(mx,my) = mom2(mx,my) + fct * abs(mom(mx,my,iz))**2
        end do
      end do
    end do

    write( cimom, fmt="(i1.1)" ) imom
    write( cis, fmt="(i1.1)" ) is
    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/mominkxky_mom"//cimom//"s"//cis//"_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "#   is=",is,   ", imom=",imom
      write( omominkxky, "(99a17)" ) "#              kx","ky","<|mom|^2>"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my), mom2(mx,my)
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE mominkxky


SUBROUTINE fluxinkxky( is, loop )
!-------------------------------------------------------------------------------
!
!     Output flux in (kx,ky) at loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_loop, rb_Al_loop, rb_mom_isloop
  use diag_geom, only : kx, gky, grootg, cfsrf

  integer, intent(in) :: is, loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: phi, Al
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1,0:nmom-1) :: mom
  real(kind=DP), dimension(-nx:nx,0:global_ny) :: pflux_es, pflux_em, eflux_es, eflux_em
  character(len=1) :: cis
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz

    call rb_phi_gettime( loop, time )
    call rb_phi_loop( loop, phi )
    call rb_Al_loop( loop, Al )
    call rb_mom_isloop( is, loop, mom )

    pflux_es(:,:) = 0._DP
    pflux_em(:,:) = 0._DP
    eflux_es(:,:) = 0._DP
    eflux_em(:,:) = 0._DP
    do iz = -global_nz, global_nz-1
      fct = grootg(iz) / cfsrf      ! flux-surface average
      do my = 0, global_ny
        do mx = -nx, nx
          pflux_es(mx,my) = pflux_es(mx,my) + fct  &
                          * real( - ui * gky(my) * phi(mx,my,iz)  &
                                  * conjg( mom(mx,my,iz,0) ), kind=DP )
          pflux_em(mx,my) = pflux_em(mx,my) + fct  &
                          * real( ui * gky(my) * Al(mx,my,iz)  &
                                  * conjg( mom(mx,my,iz,1) ), kind=DP )
          eflux_es(mx,my) = eflux_es(mx,my) + fct  &
                          * real( - ui * gky(my) * phi(mx,my,iz) &
                                  * conjg( mom(mx,my,iz,2)   &
                                         + mom(mx,my,iz,3) ), kind=DP )
          eflux_em(mx,my) = eflux_em(mx,my) + fct  &
                          * real( ui * gky(my) * Al(mx,my,iz)  &
                                  * conjg( mom(mx,my,iz,4) &
                                         + mom(mx,my,iz,5) ), kind=DP )
        end do
      end do
    end do

    write( cis, fmt="(i1.1)" ) is
    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/fluxinkxky_s"//cis//"_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(a,i17)" ) "#   is=",is
      write( omominkxky, "(99a17)" ) "#              kx","ky","G_E","G_M","Q_E","Q_M"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my),  &
                                             pflux_es(mx,my), pflux_em(mx,my), &
                                             eflux_es(mx,my), eflux_em(mx,my)
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE fluxinkxky


SUBROUTINE mominkxky_all( loop )
!-------------------------------------------------------------------------------
!
!     Output flux in (kx,ky) at loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_loop, rb_Al_loop, rb_mom_loop
  use diag_geom, only : kx, gky, grootg, cfsrf, fct_e_energy, fct_m_energy

  integer, intent(in) :: loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: phi, Al
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1,0:nmom-1,0:ns-1) :: mom
  real(kind=DP), dimension(-nx:nx,0:global_ny) :: e_energy, m_energy
  real(kind=DP), dimension(-nx:nx,0:global_ny,0:ns-1) :: dens, upara, pres, qpara
  real(kind=DP), dimension(-nx:nx,0:global_ny,0:ns-1) :: pflux_es, pflux_em, eflux_es, eflux_em
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz, is

    call rb_phi_gettime( loop, time )
    call rb_phi_loop( loop, phi )
    call rb_Al_loop( loop, Al )
    call rb_mom_loop( loop, mom )

      e_energy(:,:) = 0._DP
      m_energy(:,:) = 0._DP
    do iz = -global_nz, global_nz-1
      fct = grootg(iz) / cfsrf      ! flux-surface average
      do my = 0, global_ny
        do mx = -nx, nx
          e_energy(mx,my) = e_energy(mx,my)  &
                          + fct * fct_e_energy(mx,my,iz) * abs(phi(mx,my,iz))**2
          m_energy(mx,my) = m_energy(mx,my)  &
                          + fct * fct_m_energy(mx,my,iz) * abs(Al(mx,my,iz))**2
        end do
      end do
    end do
        dens(:,:,:) = 0._DP
       upara(:,:,:) = 0._DP
        pres(:,:,:) = 0._DP
       qpara(:,:,:) = 0._DP
    pflux_es(:,:,:) = 0._DP
    pflux_em(:,:,:) = 0._DP
    eflux_es(:,:,:) = 0._DP
    eflux_em(:,:,:) = 0._DP
    do is = 0, ns-1
      do iz = -global_nz, global_nz-1
        fct = grootg(iz) / cfsrf      ! flux-surface average
        do my = 0, global_ny
          do mx = -nx, nx
                dens(mx,my,is) = dens(mx,my,is) + fct  &
                               * abs(mom(mx,my,iz,0,is))**2
               upara(mx,my,is) = upara(mx,my,is) + fct  &
                               * abs(mom(mx,my,iz,1,is))**2
                pres(mx,my,is) = pres(mx,my,is) + fct  &
                               * abs(mom(mx,my,iz,2,is)+mom(mx,my,iz,3,is))**2
               qpara(mx,my,is) = qpara(mx,my,is) + fct  &
                               * abs(mom(mx,my,iz,4,is)+mom(mx,my,iz,5,is))**2
            pflux_es(mx,my,is) = pflux_es(mx,my,is) + fct  &
                               * real( - ui * gky(my) * phi(mx,my,iz)  &
                                       * conjg( mom(mx,my,iz,0,is) ), kind=DP )
            pflux_em(mx,my,is) = pflux_em(mx,my,is) + fct  &
                               * real( ui * gky(my) * Al(mx,my,iz)  &
                                       * conjg( mom(mx,my,iz,1,is) ), kind=DP )
            eflux_es(mx,my,is) = eflux_es(mx,my,is) + fct  &
                               * real( - ui * gky(my) * phi(mx,my,iz) &
                                       * conjg( mom(mx,my,iz,2,is)    &
                                              + mom(mx,my,iz,3,is) ), kind=DP )
            eflux_em(mx,my,is) = eflux_em(mx,my,is) + fct  &
                               * real( ui * gky(my) * Al(mx,my,iz)  &
                                       * conjg( mom(mx,my,iz,4,is)  &
                                              + mom(mx,my,iz,5,is) ), kind=DP )
          end do
        end do
      end do
    end do

    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/mominkxky_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(99a17)" ) "#              kx","ky","W_E","W_M",  &
                                     "dens_s","upara_s","pres_s","qpara_s", &
                                     "G_sE","G_sM","Q_sE","Q_sM"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my),    &
                                             e_energy(mx,my),    &
                                             m_energy(mx,my),    &
                                           (     dens(mx,my,is), &
                                                upara(mx,my,is), &
                                                 pres(mx,my,is), &
                                                qpara(mx,my,is), &
                                             pflux_es(mx,my,is), &
                                             pflux_em(mx,my,is), &
                                             eflux_es(mx,my,is), &
                                             eflux_em(mx,my,is), is=0, ns-1 )
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE mominkxky_all


SUBROUTINE fpintinkxky( loop )
!-------------------------------------------------------------------------------
!
!     Output field-particle interaction terms in (kx,ky) at loop
!                                                   (S. Maeyama, 17 Sep 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_loop, rb_Al_loop, rb_mom_imomisloop, loop_phi_sta, loop_phi_end
  use diag_geom, only : kx, gky, grootg, cfsrf, sgn, Znum

  integer, intent(in) :: loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: phi, dAldt
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1,0:ns-1) :: dndt, upara
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1,-2:2) :: wk
  real(kind=DP), dimension(-nx:nx,0:global_ny,0:ns-1) :: fpint_es, fpint_em
  character(len=8) :: cloop
  real(kind=DP) :: fct
  integer :: mx, my, iz, is

    call rb_phi_gettime( loop, time )
    call rb_phi_loop( loop, phi )
    do is = 0, ns-1
      call rb_mom_imomisloop( 1, is, loop, upara(:,:,:,is) )
    end do

    if (loop == loop_phi_sta(snum)) then
      fct = 1._DP / (2._DP * dtout_ptn)
      call rb_Al_loop( loop  , wk(:,:,:, 0) )
      call rb_Al_loop( loop+1, wk(:,:,:,+1) )
      call rb_Al_loop( loop+2, wk(:,:,:,+2) )
      do iz = -global_nz, global_nz-1
        do my = 0, global_ny
          do mx = -nx, nx
            dAldt(mx,my,iz) = fct * ( -3._DP * wk(mx,my,iz, 0)  &
                                      +4._DP * wk(mx,my,iz,+1)  &
                                      -        wk(mx,my,iz,+2) )
          end do
        end do
      end do
      do is = 0, ns-1
        call rb_mom_imomisloop( 0, is, loop  , wk(:,:,:, 0) )
        call rb_mom_imomisloop( 0, is, loop+1, wk(:,:,:,+1) )
        call rb_mom_imomisloop( 0, is, loop+2, wk(:,:,:,+2) )
        do iz = -global_nz, global_nz-1
          do my = 0, global_ny
            do mx = -nx, nx
              dndt(mx,my,iz,is) = fct * ( -3._DP * wk(mx,my,iz, 0)  &
                                          +4._DP * wk(mx,my,iz,+1)  &
                                          -        wk(mx,my,iz,+2) )
            end do
          end do
        end do
      end do
    else if (loop == loop_phi_sta(snum)+1) then
      fct = 1._DP / (6._DP * dtout_ptn)
      call rb_Al_loop( loop-1, wk(:,:,:,-1) )
      call rb_Al_loop( loop  , wk(:,:,:, 0) )
      call rb_Al_loop( loop+1, wk(:,:,:,+1) )
      call rb_Al_loop( loop+2, wk(:,:,:,+2) )
      do iz = -global_nz, global_nz-1
        do my = 0, global_ny
          do mx = -nx, nx
            dAldt(mx,my,iz) = fct * ( -2._DP * wk(mx,my,iz,-1)  &
                                      -3._DP * wk(mx,my,iz, 0)  &
                                      +6._DP * wk(mx,my,iz,+1)  &
                                      -        wk(mx,my,iz,+2) )
          end do
        end do
      end do
      do is = 0, ns-1
        call rb_mom_imomisloop( 0, is, loop-1, wk(:,:,:,-1) )
        call rb_mom_imomisloop( 0, is, loop  , wk(:,:,:, 0) )
        call rb_mom_imomisloop( 0, is, loop+1, wk(:,:,:,+1) )
        call rb_mom_imomisloop( 0, is, loop+2, wk(:,:,:,+2) )
        do iz = -global_nz, global_nz-1
          do my = 0, global_ny
            do mx = -nx, nx
              dndt(mx,my,iz,is) = fct * ( -2._DP * wk(mx,my,iz,-1)  &
                                          -3._DP * wk(mx,my,iz, 0)  &
                                          +6._DP * wk(mx,my,iz,+1)  &
                                          -        wk(mx,my,iz,+2) )
            end do
          end do
        end do
      end do
    else if (loop == loop_phi_end(enum)-1) then
      fct = 1._DP / (6._DP * dtout_ptn)
      call rb_Al_loop( loop-2, wk(:,:,:,-2) )
      call rb_Al_loop( loop-1, wk(:,:,:,-1) )
      call rb_Al_loop( loop  , wk(:,:,:, 0) )
      call rb_Al_loop( loop+1, wk(:,:,:,+1) )
      do iz = -global_nz, global_nz-1
        do my = 0, global_ny
          do mx = -nx, nx
            dAldt(mx,my,iz) = fct * ( +        wk(mx,my,iz,-2)  &
                                      -6._DP * wk(mx,my,iz,-1)  &
                                      +3._DP * wk(mx,my,iz, 0)  &
                                      +2._DP * wk(mx,my,iz,+1) )
          end do
        end do
      end do
      do is = 0, ns-1
        call rb_mom_imomisloop( 0, is, loop-2, wk(:,:,:,-2) )
        call rb_mom_imomisloop( 0, is, loop-1, wk(:,:,:,-1) )
        call rb_mom_imomisloop( 0, is, loop  , wk(:,:,:, 0) )
        call rb_mom_imomisloop( 0, is, loop+1, wk(:,:,:,+1) )
        do iz = -global_nz, global_nz-1
          do my = 0, global_ny
            do mx = -nx, nx
              dndt(mx,my,iz,is) = fct * ( +        wk(mx,my,iz,-2)  &
                                          -6._DP * wk(mx,my,iz,-1)  &
                                          +3._DP * wk(mx,my,iz, 0)  &
                                          +2._DP * wk(mx,my,iz,+1) )
            end do
          end do
        end do
      end do
    else if (loop == loop_phi_end(enum)) then
      fct = 1._DP / (2._DP * dtout_ptn)
      call rb_Al_loop( loop-2, wk(:,:,:,-2) )
      call rb_Al_loop( loop-1, wk(:,:,:,-1) )
      call rb_Al_loop( loop  , wk(:,:,:, 0) )
      do iz = -global_nz, global_nz-1
        do my = 0, global_ny
          do mx = -nx, nx
            dAldt(mx,my,iz) = fct * ( +        wk(mx,my,iz,-2)  &
                                      -4._DP * wk(mx,my,iz,-1)  &
                                      +3._DP * wk(mx,my,iz, 0) )
          end do
        end do
      end do
      do is = 0, ns-1
        call rb_mom_imomisloop( 0, is, loop-2, wk(:,:,:,-2) )
        call rb_mom_imomisloop( 0, is, loop-1, wk(:,:,:,-1) )
        call rb_mom_imomisloop( 0, is, loop  , wk(:,:,:, 0) )
        do iz = -global_nz, global_nz-1
          do my = 0, global_ny
            do mx = -nx, nx
              dndt(mx,my,iz,is) = fct * ( +        wk(mx,my,iz,-2)  &
                                          -4._DP * wk(mx,my,iz,-1)  &
                                          +3._DP * wk(mx,my,iz, 0) )
            end do
          end do
        end do
      end do
    else ! loop_phi_sta(snum)+2 <= loop <= loop_phi_end(enum)
      fct = 1._DP / (12._DP * dtout_ptn)
      call rb_Al_loop( loop-2, wk(:,:,:,-2) )
      call rb_Al_loop( loop-1, wk(:,:,:,-1) )
      call rb_Al_loop( loop+1, wk(:,:,:,+1) )
      call rb_Al_loop( loop+2, wk(:,:,:,+2) )
      do iz = -global_nz, global_nz-1
        do my = 0, global_ny
          do mx = -nx, nx
            dAldt(mx,my,iz) = fct * ( +        wk(mx,my,iz,-2)  &
                                      -8._DP * wk(mx,my,iz,-1)  &
                                      +8._DP * wk(mx,my,iz,+1)  &
                                      -        wk(mx,my,iz,+2) )
          end do
        end do
      end do
      do is = 0, ns-1
        call rb_mom_imomisloop( 0, is, loop-2, wk(:,:,:,-2) )
        call rb_mom_imomisloop( 0, is, loop-1, wk(:,:,:,-1) )
        call rb_mom_imomisloop( 0, is, loop+1, wk(:,:,:,+1) )
        call rb_mom_imomisloop( 0, is, loop+2, wk(:,:,:,+2) )
        do iz = -global_nz, global_nz-1
          do my = 0, global_ny
            do mx = -nx, nx
              dndt(mx,my,iz,is) = fct * ( +        wk(mx,my,iz,-2)  &
                                          -8._DP * wk(mx,my,iz,-1)  &
                                          +8._DP * wk(mx,my,iz,+1)  &
                                          -        wk(mx,my,iz,+2) )
            end do
          end do
        end do
      end do
    end if

    fpint_es(:,:,:) = 0._DP
    fpint_em(:,:,:) = 0._DP
    do is = 0, ns-1
      do iz = -global_nz, global_nz-1
        fct = grootg(iz) / cfsrf      ! flux-surface average
        do my = 0, global_ny
          do mx = -nx, nx
            fpint_es(mx,my,is) = fpint_es(mx,my,is) + fct  &
                               * real( - conjg( phi(mx,my,iz) ) * sgn(is) * Znum(is) &
                                       * dndt(mx,my,iz,is), kind=DP )
            fpint_em(mx,my,is) = fpint_em(mx,my,is) + fct  &
                               * real( - conjg( upara(mx,my,iz,is) ) * sgn(is) * Znum(is) &
                                       * dAldt(mx,my,iz), kind=DP )
          end do
        end do
      end do
    end do

    write( cloop, fmt="(i8.8)" ) loop
    open( omominkxky, file="./data/fpintinkxky_t"//cloop//".dat" )
      write( omominkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominkxky, "(99a17)" ) "#              kx","ky","R_sE", "R_sM"
      do my = 0, global_ny
        do mx = -nx, nx
          write( omominkxky, "(99g17.7e3)" ) kx(mx), gky(my),    &
                                           ( fpint_es(mx,my,is), &
                                             fpint_em(mx,my,is), is=0, ns-1 )
        end do
        write( omominkxky, * )
      end do
    close( omominkxky )

END SUBROUTINE fpintinkxky


END MODULE out_mominkxky
