MODULE out_mominvtk
!-------------------------------------------------------------------------------
!
!     Output moments in (x,y,z)
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_header
  implicit none

  private

  public phiinvtk, Alinvtk, mominvtk


 CONTAINS


SUBROUTINE phiinvtk( flag, loop_sta, loop_end, loop_skip )
!-------------------------------------------------------------------------------
!
!     Output phi in (xx,yy,zz,time) in VTK binary format
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_loop
  use diag_geom, only : lx, ly, dz, q_0, s_hat, r_minor, eps_r, ck, dj, n_alp, n_tht
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: flag ! 1:coord&var&header(fluxtube)
                              ! 3:coord&var&header(fulltorus)
                              ! 5:coord&var&header(fieldalignedcoord)
  integer, intent(in) :: loop_sta, loop_end, loop_skip

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: phi
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: wc2
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1) :: wr2
  real(kind=DP) :: wxx, wyy, wzz, wsr, wmr, wdx, wdy, wtheta, wzeta, wq_bar, wx_car, wy_car, wz_car
  character(len=8) :: cloop
  integer :: mx, my, iz, loop, mwp, i_alp

    if ( flag == 1 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_phi_gettime( loop, time )

        open( omominxyz, file="./data/phiinvtk_tube_t"//cloop//".vtk", status="replace",  &
                         action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do my = 0, 2*nyw
            wyy = -ly + wdy * real( my, kind=DP )
            do mx = 0, 2*nxw
              wxx = -lx + wdx * real( mx, kind=DP )
              wsr = r_minor + wxx
              wmr = r_minor / eps_r + wsr * cos( wtheta )
              !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wx_car = wmr * cos( wzeta )
              wy_car = wmr * sin( wzeta )
              wz_car = wsr * sin( wtheta )
              write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                               real(wy_car, kind=4),  &
                                               real(wz_car, kind=4)
            end do
          end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS phi float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_phi_loop( loop, phi )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( phi(:,:,iz), wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * phi(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 3 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_phi_gettime( loop, time )

        open( omominxyz, file="./data/phiinvtk_full_t"//cloop//".vtk", status="replace",  &
                         action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw*n_alp+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
            end do
          end do
          i_alp = n_alp-1
            my = 2*nyw
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS phi float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_phi_loop( loop, phi )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( phi(:,:,iz), wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * phi(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 5 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_phi_gettime( loop, time )

        open( omominxyz, file="./data/phiinvtk_align_t"//cloop//".vtk_head", status="replace",  &
                         action="write", form="formatted", access="sequential" )
          write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
          write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                            "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
          write( omominxyz, '(a)' ) "BINARY"
          write( omominxyz, '(a)' ) "DATASET STRUCTURED_POINTS"
          write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw, 2*nyw, 2*global_nz
          write( omominxyz, '(a)' ) "ORIGIN 0.0 0.0 0.0"
          write( omominxyz, '(a,3g17.7)' ) "SPACING ", lx/real(nxw), ly/real(nyw), 4._DP*lx/real(global_nz)
          write( omominxyz, * )
          write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw)*(2*nyw)*(2*global_nz)
          write( omominxyz, '(a)' ) "SCALARS phi float 1"
          write( omominxyz, '(a)' ) "LOOKUP_TABLE default 1"
        close( omominxyz )
        open( omominxyz, file="./data/phiinvtk_align_t"//cloop//".vtk_bin", status="replace",  &
                         !action="write", form="binary", convert="BIG_ENDIAN" )
                         action="write", form="unformatted", access="stream", convert="BIG_ENDIAN" )
          call rb_phi_loop( loop, phi )
          do iz = -global_nz, global_nz-1
            call fft_backward_xy( phi(:,:,iz), wr2 )
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz ) real(wr2(mx,my), kind=4)
              end do
            end do
          end do
        close( omominxyz )

      end do

    else

      write(*,*) "flag in phiinvtk is invalid !  flag =", flag
      stop

    end if

END SUBROUTINE phiinvtk


SUBROUTINE Alinvtk( flag, loop_sta, loop_end, loop_skip )
!-------------------------------------------------------------------------------
!
!     Output Al in (xx,yy,zz,time) in VTK binary format
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_Al_gettime, rb_Al_loop
  use diag_geom, only : lx, ly, dz, q_0, s_hat, r_minor, eps_r, ck, dj, n_alp, n_tht
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: flag ! 1:coord&var&header(fluxtube)
                              ! 3:coord&var&header(fulltorus)
                              ! 5:coord&var&header(fieldalignedcoord)
  integer, intent(in) :: loop_sta, loop_end, loop_skip

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: Al
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: wc2
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1) :: wr2
  real(kind=DP) :: wxx, wyy, wzz, wsr, wmr, wdx, wdy, wtheta, wzeta, wq_bar, wx_car, wy_car, wz_car
  character(len=8) :: cloop
  integer :: mx, my, iz, loop, mwp, i_alp

    if ( flag == 1 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_Al_gettime( loop, time )

        open( omominxyz, file="./data/Alinvtk_tube_t"//cloop//".vtk", status="replace",  &
                         action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do my = 0, 2*nyw
            wyy = -ly + wdy * real( my, kind=DP )
            do mx = 0, 2*nxw
              wxx = -lx + wdx * real( mx, kind=DP )
              wsr = r_minor + wxx
              wmr = r_minor / eps_r + wsr * cos( wtheta )
              !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wx_car = wmr * cos( wzeta )
              wy_car = wmr * sin( wzeta )
              wz_car = wsr * sin( wtheta )
              write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                               real(wy_car, kind=4),  &
                                               real(wz_car, kind=4)
            end do
          end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS Al float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_Al_loop( loop, Al )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( Al(:,:,iz), wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * Al(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 3 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_Al_gettime( loop, time )

        open( omominxyz, file="./data/Alinvtk_full_t"//cloop//".vtk", status="replace",  &
                         action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw*n_alp+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
            end do
          end do
          i_alp = n_alp-1
            my = 2*nyw
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS Al float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_Al_loop( loop, Al )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( Al(:,:,iz), wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * Al(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 5 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_Al_gettime( loop, time )

        open( omominxyz, file="./data/Alinvtk_align_t"//cloop//".vtk_head", status="replace",  &
                         action="write", form="formatted", access="sequential" )
          write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
          write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                            "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
          write( omominxyz, '(a)' ) "BINARY"
          write( omominxyz, '(a)' ) "DATASET STRUCTURED_POINTS"
          write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw, 2*nyw, 2*global_nz
          write( omominxyz, '(a)' ) "ORIGIN 0.0 0.0 0.0"
          write( omominxyz, '(a,3g17.7)' ) "SPACING ", lx/real(nxw), ly/real(nyw), 4._DP*lx/real(global_nz)
          write( omominxyz, * )
          write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw)*(2*nyw)*(2*global_nz)
          write( omominxyz, '(a)' ) "SCALARS Al float 1"
          write( omominxyz, '(a)' ) "LOOKUP_TABLE default 1"
        close( omominxyz )
        open( omominxyz, file="./data/Alinvtk_align_t"//cloop//".vtk_bin", status="replace",  &
                         !action="write", form="binary", convert="BIG_ENDIAN" )
                         action="write", form="unformatted", access="stream", convert="BIG_ENDIAN" )
          call rb_Al_loop( loop, Al )
          do iz = -global_nz, global_nz-1
            call fft_backward_xy( Al(:,:,iz), wr2 )
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz ) real(wr2(mx,my), kind=4)
              end do
            end do
          end do
        close( omominxyz )

      end do

    else

      write(*,*) "flag in Alinvtk is invalid !  flag =", flag
      stop

    end if

END SUBROUTINE Alinvtk


SUBROUTINE mominvtk( flag, imom, is, loop_sta, loop_end, loop_skip )
!-------------------------------------------------------------------------------
!
!     Output mom in (xx,yy,zz,time) in VTK binary format
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_mom_gettime, rb_mom_imomisloop
  use diag_geom, only : lx, ly, dz, q_0, s_hat, r_minor, eps_r, ck, dj, n_alp, n_tht
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: flag ! 1:coord&var&header(fluxtube)
                              ! 3:coord&var&header(fulltorus)
                              ! 5:coord&var&header(fieldalignedcoord)
  integer, intent(in) :: imom, is, loop_sta, loop_end, loop_skip

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny,-global_nz:global_nz-1) :: mom
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: wc2
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1) :: wr2
  real(kind=DP) :: wxx, wyy, wzz, wsr, wmr, wdx, wdy, wtheta, wzeta, wq_bar, wx_car, wy_car, wz_car
  character(len=8) :: cloop
  character(len=1) :: cimom, cis
  integer :: mx, my, iz, loop, mwp, i_alp

    write( cimom, '(i1.1)' ) imom
    write( cis, '(i1.1)' ) is

    if ( flag == 1 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_mom_gettime( loop, time )

        open( omominxyz, file="./data/mominvtk_tube_mom"//cimom//"s"//cis//"_t"//cloop//".vtk", &
                         status="replace", action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do my = 0, 2*nyw
            wyy = -ly + wdy * real( my, kind=DP )
            do mx = 0, 2*nxw
              wxx = -lx + wdx * real( mx, kind=DP )
              wsr = r_minor + wxx
              wmr = r_minor / eps_r + wsr * cos( wtheta )
              !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
              wx_car = wmr * cos( wzeta )
              wy_car = wmr * sin( wzeta )
              wz_car = wsr * sin( wtheta )
              write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                               real(wy_car, kind=4),  &
                                               real(wz_car, kind=4)
            end do
          end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS mom"//cimom//cis//" float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_mom_imomisloop( imom, is, loop, mom )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( mom(:,:,iz), wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * mom(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do my = 0, 2*nyw-1
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
          end do
          ! my = 2*nyw
            do mx = 0, 2*nxw-1
              write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
            end do
            ! mx = 2*nxw
            write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 3 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_mom_gettime( loop, time )

        open( omominxyz, file="./data/mominvtk_full_mom"//cimom//"s"//cis//"_t"//cloop//".vtk", &
                         status="replace", action="write", form="formatted" )

        write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
        write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                          "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
        write( omominxyz, '(a)' ) "ASCII"
        write( omominxyz, '(a)' ) "DATASET STRUCTURED_GRID"
        write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw+1, 2*nyw*n_alp+1, 2*global_nz+1
        write( omominxyz, '(a,i17,a)' ) "POINTS ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1), " float"
        wdx = lx / real( nxw, kind=DP )
        wdy = ly / real( nyw, kind=DP )
        wq_bar = q_0 * sqrt( 1._DP - eps_r**2 )
        do iz = -global_nz, global_nz
          wzz = dz * real( iz, kind=DP )
          !wtheta = 2._DP * atan( sqrt( (1._DP+eps_r)/(1._DP-eps_r) ) * tan(0.5_DP*wzz) )
          wtheta = wzz
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
            end do
          end do
          i_alp = n_alp-1
            my = 2*nyw
              wyy = -ly + wdy * real( my, kind=DP )
              do mx = 0, 2*nxw
                wxx = -lx + wdx * real( mx, kind=DP )
                wsr = r_minor + wxx
                wmr = r_minor / eps_r + wsr * cos( wtheta )
                !wzeta = wq_bar * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )
                !      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wzeta = q_0 * ( wzz + ( s_hat * wxx * wzz - wyy ) / r_minor )  &
                      - 2._DP * pi * real( i_alp, kind=DP ) / real( n_alp, kind=DP )
                wx_car = wmr * cos( wzeta )
                wy_car = wmr * sin( wzeta )
                wz_car = wsr * sin( wtheta )
                write( omominxyz, '(3g17.7e3)' ) real(wx_car, kind=4),  &
                                                 real(wy_car, kind=4),  &
                                                 real(wz_car, kind=4)
              end do
        end do

        write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw+1)*(2*nyw*n_alp+1)*(2*global_nz+1)
        write( omominxyz, '(a)' ) "SCALARS mom"//cimom//cis//" float 1"
        write( omominxyz, '(a)' ) "LOOKUP_TABLE default"
        call rb_mom_imomisloop( imom, is, loop, mom )
        do iz = -global_nz, global_nz-1
          call fft_backward_xy( mom(:,:,iz), wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)
        end do
        ! iz = global_nz
          do my = 0, global_ny
            do mx = -nx, nx
              mwp = mx - dj(my)          ! --- mw = mx - dj for the positive-z 
                if( abs(mwp) > nx ) then
                  wc2(mx,my) = ( 0._DP, 0._DP )
                else
                  wc2(mx,my) = conjg( ck(my) ) * mom(mwp,my,-global_nz)
                end if
            end do
          end do
          call fft_backward_xy( wc2, wr2 )
          do i_alp = 0, n_alp-1
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,my), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,my), kind=4)
            end do
          end do
            ! my = 2*nyw
              do mx = 0, 2*nxw-1
                write( omominxyz, '(g17.7e3)' ) real(wr2(mx,0), kind=4)
              end do
              ! mx = 2*nxw
              write( omominxyz, '(g17.7e3)' ) real(wr2(0,0), kind=4)

        close( omominxyz )

      end do

    else if ( flag == 5 ) then ! output variable

      do loop = loop_sta, loop_end, loop_skip
        write( cloop, '(i8.8)' ) loop
        call rb_mom_gettime( loop, time )

        
        open( omominxyz, file="./data/mominvtk_align_mom"//cimom//"s"//cis//"_t"//cloop//".vtk_head", &
                         status="replace", action="write", form="formatted", access="sequential" )
          write( omominxyz, '(a)' ) "# vtk DataFile Version 2.0"
          write( omominxyz, '(a,g17.7e3,a,g17.7e3,a,g17.7e3,a,g17.7e3)' )  &
                            "time= ", time, " lx= ", lx, " ly= ", ly, " lz= ", pi * real( n_tht, kind=DP )
          write( omominxyz, '(a)' ) "BINARY"
          write( omominxyz, '(a)' ) "DATASET STRUCTURED_POINTS"
          write( omominxyz, '(a,3i17)' ) "DIMENSIONS ", 2*nxw, 2*nyw, 2*global_nz
          write( omominxyz, '(a)' ) "ORIGIN 0.0 0.0 0.0"
          write( omominxyz, '(a,3g17.7)' ) "SPACING ", lx/real(nxw), ly/real(nyw), 4._DP*lx/real(global_nz)
          write( omominxyz, * )
          write( omominxyz, '(a,i17)' ) "POINT_DATA ", (2*nxw)*(2*nyw)*(2*global_nz)
          write( omominxyz, '(a)' ) "SCALARS mom"//cimom//cis//" float 1"
          write( omominxyz, '(a)' ) "LOOKUP_TABLE default 1"
        close( omominxyz )
        open( omominxyz, file="./data/mominvtk_align_mom"//cimom//"s"//cis//"_t"//cloop//".vtk_bin", &
                         !status="replace", action="write", form="binary", convert="BIG_ENDIAN" )
                         status="replace", action="write", form="unformatted", access="stream", convert="BIG_ENDIAN" )
          call rb_mom_imomisloop( imom, is, loop, mom )
          do iz = -global_nz, global_nz-1
            call fft_backward_xy( mom(:,:,iz), wr2 )
            do my = 0, 2*nyw-1
              do mx = 0, 2*nxw-1
                write( omominxyz ) real(wr2(mx,my), kind=4)
              end do
            end do
          end do
        close( omominxyz )

      end do

    else

      write(*,*) "flag in mominvtk is invalid !  flag =", flag
      stop

    end if

END SUBROUTINE mominvtk


END MODULE out_mominvtk
