MODULE out_mominxy
!-------------------------------------------------------------------------------
!
!     Output moments in (x,y)
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_header
  implicit none

  private

  public phiinxy, Alinxy, mominxy


 CONTAINS


SUBROUTINE phiinxy( giz, loop )
!-------------------------------------------------------------------------------
!
!     Output phi in (xx,yy) at giz, loop
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_phi_gettime, rb_phi_izloop
  use diag_geom, only : xx, yy, gzz
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: giz, loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: phikxky
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1)   :: phixy
  character(len=4) :: ciz
  character(len=8) :: cloop
  integer :: mx, my

    call rb_phi_gettime( loop, time )
    call rb_phi_izloop( giz, loop, phikxky )
    call fft_backward_xy( phikxky, phixy )

    write( ciz, fmt="(i4.4)" ) giz
    write( cloop, fmt="(i8.8)" ) loop
    open( omominxy, file="./data/phiinxy_z"//ciz//"_t"//cloop//".dat" )
      write( omominxy, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominxy, "(a,i17,a,g17.7e3)" ) "#  giz=",giz,  ",   zz=",gzz(giz)
      write( omominxy, "(99a17)" ) "#              xx","yy","phi"
      do my = 0, 2*nyw-1
        do mx = 0, 2*nxw-1
          write( omominxy, "(99g17.7e3)" ) xx(mx), yy(my), phixy(mx,my)
        end do
        write( omominxy, * )
      end do
    close( omominxy )

END SUBROUTINE phiinxy


SUBROUTINE Alinxy( giz, loop )
!-------------------------------------------------------------------------------
!
!     Output Al in (xx,yy) at giz, loop
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_Al_gettime, rb_Al_izloop
  use diag_geom, only : xx, yy, gzz
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: giz, loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: Alkxky
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1)   :: Alxy
  character(len=4) :: ciz
  character(len=8) :: cloop
  integer :: mx, my

    call rb_Al_gettime( loop, time )
    call rb_Al_izloop( giz, loop, Alkxky )
    call fft_backward_xy( Alkxky, Alxy )

    write( ciz, fmt="(i4.4)" ) giz
    write( cloop, fmt="(i8.8)" ) loop
    open( omominxy, file="./data/Alinxy_z"//ciz//"_t"//cloop//".dat" )
      write( omominxy, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominxy, "(a,i17,a,g17.7e3)" ) "#  giz=",giz,  ",   zz=",gzz(giz)
      write( omominxy, "(99a17)" ) "#              xx","yy","Al"
      do my = 0, 2*nyw-1
        do mx = 0, 2*nxw-1
          write( omominxy, "(99g17.7e3)" ) xx(mx), yy(my), Alxy(mx,my)
        end do
        write( omominxy, * )
      end do
    close( omominxy )

END SUBROUTINE Alinxy


SUBROUTINE mominxy( giz, imom, is, loop )
!-------------------------------------------------------------------------------
!
!     Output moments in (xx,yy) at giz, imom, is, loop
!                                                   (S. Maeyama, 11 June 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_mom_gettime, rb_mom_izimomisloop
  use diag_geom, only : xx, yy, gzz
  use diag_fft, only : fft_backward_xy

  integer, intent(in) :: giz, imom, is, loop

  real(kind=DP) :: time
  complex(kind=DP), dimension(-nx:nx,0:global_ny) :: momkxky
  real(kind=DP), dimension(0:2*nxw-1,0:2*nyw-1)   :: momxy
  character(len=4) :: ciz
  character(len=1) :: cimom, cis
  character(len=8) :: cloop
  integer :: mx, my

    call rb_mom_gettime( loop, time )
    call rb_mom_izimomisloop( giz, imom, is, loop, momkxky )
    call fft_backward_xy( momkxky, momxy )

    write( ciz, fmt="(i4.4)" ) giz
    write( cimom, fmt="(i1.1)" ) imom
    write( cis, fmt="(i1.1)" ) is
    write( cloop, fmt="(i8.8)" ) loop
    open( omominxy, file="./data/mominxy_z"//ciz//"mom"//cimom//"s"//cis//"_t"//cloop//".dat" )
      write( omominxy, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( omominxy, "(a,i17,a,g17.7e3)" ) "#   is=",is,   ", imom=",imom
      write( omominxy, "(a,i17,a,g17.7e3)" ) "#  giz=",giz,  ",   zz=",gzz(giz)
      write( omominxy, "(99a17)" ) "#              xx","yy","mom"
      do my = 0, 2*nyw-1
        do mx = 0, 2*nxw-1
          write( omominxy, "(99g17.7e3)" ) xx(mx), yy(my), momxy(mx,my)
        end do
        write( omominxy, * )
      end do
    close( omominxy )

END SUBROUTINE mominxy


END MODULE out_mominxy
