MODULE out_trninkxky
!-------------------------------------------------------------------------------
!
!     Output transfer diagnostics in (kx,ky)
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_header
  implicit none

  private

  public trninkxky, trninkxky_all


 CONTAINS


SUBROUTINE trninkxky( is, loop )
!-------------------------------------------------------------------------------
!
!     Output transfer diagnostics in (kx,ky) at itrn, is, loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_trn_gettime, rb_trn_isloop
  use diag_geom, only : kx, gky

  integer, intent(in) :: is, loop

  real(kind=DP) :: time
  real(kind=DP), dimension(-nx:nx,0:global_ny,0:ntrn-1) :: trn
  character(len=1) :: cis
  character(len=8) :: cloop
  integer :: mx, my

    call rb_trn_gettime( loop, time )
    call rb_trn_isloop( is, loop, trn )

    write( cis, fmt="(i1.1)" ) is
    write( cloop, fmt="(i8.8)" ) loop
    open( otrninkxky, file="./data/trninkxky_s"//cis//"_t"//cloop//".dat" )
      write( otrninkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( otrninkxky, "(a,i17)" ) "#   is=",is
      write( otrninkxky, "(99a17)" ) "#              kx","ky","entropy","neint","nmint","col_disp"
      do my = 0, global_ny
        do mx = -nx, nx
          write( otrninkxky, "(99g17.7e3)" ) kx(mx), gky(my), trn(mx,my,:)
        end do
        write( otrninkxky, * )
      end do
    close( otrninkxky )

END SUBROUTINE trninkxky


SUBROUTINE trninkxky_all( loop )
!-------------------------------------------------------------------------------
!
!     Output transfer diagnostics in (kx,ky) at itrn, is, loop
!                                                   (S. Maeyama, 24 Aug 2014)
!
!-------------------------------------------------------------------------------
  use diag_rb, only : rb_trn_gettime, rb_trn_loop
  use diag_geom, only : kx, gky

  integer, intent(in) :: loop

  real(kind=DP) :: time
  real(kind=DP), dimension(-nx:nx,0:global_ny,0:ntrn-1,0:ns-1) :: trn
  character(len=8) :: cloop
  integer :: mx, my

    call rb_trn_gettime( loop, time )
    call rb_trn_loop( loop, trn )

    write( cloop, fmt="(i8.8)" ) loop
    open( otrninkxky, file="./data/trninkxky_t"//cloop//".dat" )
      write( otrninkxky, "(a,i17,a,g17.7e3)" ) "# loop=",loop, ", time=",time
      write( otrninkxky, "(99a17)" ) "#              kx","ky","entropy","neint","nmint","col_disp"
      do my = 0, global_ny
        do mx = -nx, nx
          write( otrninkxky, "(99g17.7e3)" ) kx(mx), gky(my), trn(mx,my,:,:)
        end do
        write( otrninkxky, * )
      end do
    close( otrninkxky )

END SUBROUTINE trninkxky_all


END MODULE out_trninkxky
